package com.seeyon.cip.spi.user.avoid.thirdavoid;

import com.seeyon.cip.spi.user.avoid.thirdavoid.constants.ThirdavoidSsoAvoidConstants;
import com.seeyon.cip.spi.user.avoid.thirdavoid.handler.ThirdavoidSsoAvoidHandler;
import com.seeyon.ctp.user.annotation.CtpUserChannelRouter;
import com.seeyon.ctp.user.api.avoidlogin.CtpAvoidLoginMiddlePageProviderService;
import com.seeyon.ctp.user.dto.CtpAvoidLoginUserInfoDto;
import com.seeyon.ctp.user.dto.CtpUserSpiThirdTokenDto;
import com.seeyon.ctp.user.dto.ThirdTokenDto;
import com.seeyon.ctp.user.util.CtpUserSpiUtils;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * 免登 中间页面模式接口实现<br/>
 * 注意：<br/>
 * 类中不需要加 @Service 方法， 也不能使用 @Autowired 注解， 无法获取到对应实现。需要使用 CtpUserSpiUtils#getInstance() 方法获取实例<br/>
 * request 对象获取： CtpUserSpiUtils#getRequest()<br/>
 * nacos 等配置信息获取： CtpUserSpiUtils#getPropertyByName(java.lang.String)，通过配置的 key 直接获取
 */
@Slf4j
@CtpUserChannelRouter("thirdavoid")
public class ThirdavoidSsoAvoidMiddlePageProviderService implements CtpAvoidLoginMiddlePageProviderService {

    private ThirdavoidSsoAvoidHandler thirdavoidSsoAvoidHandler = CtpUserSpiUtils.getInstance(ThirdavoidSsoAvoidHandler.class);

    /**
     * 返回客户端标识,与免登地址中 thirdType 的值一致
     */
    @Override
    public String getClientId() {
        return ThirdavoidSsoAvoidConstants.THIRD_TYPE;
    }

    /**
     * 从请求中提取参数并获取用户信息
     *
     * @param code    三方追加的 code 的值
     * @param extData 三方追加的扩展参数
     * @return 用户属性信息
     */
    @Override
    public CtpAvoidLoginUserInfoDto getUserInfo(String code, Map<String, String> extData) {
        log.info("从请求中提取参数并获取用户信息, code:{}", code);
        // 调用内部方法进行逻辑处理
        thirdavoidSsoAvoidHandler.test();
        // [按需实现] 获取 request 对象
        HttpServletRequest request = CtpUserSpiUtils.getRequest();
        // 从 request 中获取想要的参数
        request.getParameter("xxx");
        // 通过 code 获取 token
        // eg. String token = getTokenByCode(code);
        // 通过 token 获取用户登录名
        // eg. String loginName = getLoginNameByToken(token);

        //  // [按需实现] 免登成功后重定向地址, 如果不配置， 则默认跳转至打开免登时的目标地址
        // CtpUserSpiRedirectUrlDto ctpUserSpiRedirectUrlDto = CtpUserSpiRedirectUrlDto.builder()
        //       .webRedirectUrl("/main/portal") // web 重定向地址
        //       .mobileRedirectUrl("/main-mobile/portal") // mobile 重定向地址
        //       .build();

        // // [按需实现] 缓存免登后保存的三方 token
        // CtpUserSpiThirdTokenDto ctpUserSpiThirdTokenDto = CtpUserSpiThirdTokenDto.builder()
        //         .thirdToken("xxx") // 待缓存的三方 token
        //         .expired(null) // 过期时间_秒
        //         .extData(new HashMap<>()) // 扩展参数
        //         .build();

        // 返回用户信息、等缓存的三方认证 token、重定向地址
        return CtpAvoidLoginUserInfoDto.builder()
                // 用于登录的用户信息， 下面五个参数只选择一下赋值即可， 赋值的参数的值要与 v8系统中的对应的值一致
                .thirdLoginName("zhangsan")
                // .thirdUserId("123456")
                // .thirdMobile("13900000001")
                // .thirdUserEmail("abc@abc.com")
                // .thirdUserCode("usercode")
                //用于登录的用户信息， 上面五个参数只选择一下赋值即可

                // [按需实现] 登录用户的租户信息
                //.tenantType() // 租户数据类型：code / id
                //.tenantData() // 与 tenantType 对应， 租户信息的值

                // // [按需实现] 待缓存免登后保存的三方 token
                // .ctpUserSpiThirdTokenDto(ctpUserSpiThirdTokenDto) // 三方认证 token

                // [按需实现] 免登成功后重定向地址
                //.ctpUserSpiRedirectUrlDto(ctpUserSpiRedirectUrlDto); // 免登成功后重定向地址
                .build();
    }

    /**
     * [按需实现] 是否周期性刷新 token
     */
    @Override
    public Boolean isRefreshTokenCycle() {
        return null;
    }

    /**
     * 刷新三方 token
     *
     * @param thirdTokenDto 三方 token 信息
     * @return 三方 token 保存 DTO
     */
    @Override
    public CtpUserSpiThirdTokenDto doRefreshToken(ThirdTokenDto thirdTokenDto) {
        return CtpAvoidLoginMiddlePageProviderService.super.doRefreshToken(thirdTokenDto);
    }
}